@@ -3,7 +3,9 @@ package POE::Component::IRC::Common;
 use strict;
 use warnings;
 
-our $VERSION = '5.18';
+use Encode qw(decode);
+use Encode::Guess;
+our $VERSION = '6.08';
 
 require Exporter;
 use base qw(Exporter);
@@ -12,7 +14,7 @@ our @EXPORT_OK = qw(
     parse_user irc_ip_get_version irc_ip_is_ipv4 irc_ip_is_ipv6 has_color
     has_formatting strip_color strip_formatting NORMAL BOLD UNDERLINE REVERSE
     WHITE BLACK DARK_BLUE DARK_GREEN RED BROWN PURPLE ORANGE YELLOW LIGHT_GREEN
-    TEAL CYAN LIGHT_BLUE MAGENTA DARK_GREY LIGHT_GREY
+    TEAL CYAN LIGHT_BLUE MAGENTA DARK_GREY LIGHT_GREY irc_to_utf8
 );
 our %EXPORT_TAGS = ( ALL => [@EXPORT_OK] );
 
@@ -225,6 +227,12 @@ sub strip_formatting {
     return $string;
 }
 
+sub irc_to_utf8 {
+    my ($line) = @_;
+    my $utf8 = guess_encoding($line, 'utf8');
+    return ref $utf8 ? decode('utf8', $line) : decode('cp1252', $line);
+}
+
 #------------------------------------------------------------------------------
 # Subroutine ip_get_version
 # Purpose           : Get an IP version
@@ -474,8 +482,9 @@ Takes a list representing an IRC mode line. Returns a hashref. If the modeline
 couldn't be parsed the hashref will be empty. On success the following keys
 will be available in the hashref:
 
- 'modes', an arrayref of normalised modes;
- 'args', an arrayref of applicable arguments to the modes;
+B<'modes'>, an arrayref of normalised modes;
+
+B<'args'>, an arrayref of applicable arguments to the modes;
 
 Example:
 
@@ -504,7 +513,7 @@ Takes two parameters, a string representing an IRC mask ( it'll be processed
 with parse_ban_mask() to ensure that it is normalised ) and something to match
 against the IRC mask, such as a nick!user@hostname string. Returns a true
 value if they match, a false value otherwise. Optionally, one may pass the
-casemapping (see L<C<u_irc()>|/"u_irc">), as this function uses C<u_irc()>
+casemapping (see L<C<u_irc>|/"u_irc">), as this function uses C<u_irc>
 internally.
 
 =head2 C<matches_mask_array>
@@ -513,8 +522,8 @@ Takes two array references, the first being a list of strings representing
 IRC masks, the second a list of somethings to test against the masks. Returns
 an empty hashref if there are no matches. Otherwise, the keys will be the
 masks matched, each value being an arrayref of the strings that matched it.
-Optionally, one may pass the casemapping (see L<C<u_irc()>|/"u_irc">), as
-this function uses C<u_irc()> internally.
+Optionally, one may pass the casemapping (see L<C<u_irc>|/"u_irc">), as
+this function uses C<u_irc> internally.
 
 =head2 C<parse_user>
 
@@ -550,6 +559,19 @@ be cancelled with the same character, this might strip more than you hoped for
 if the string contains both color and formatting codes. Stripping both will
 always do what you expect it to.
 
+=head2 C<irc_to_utf8>
+
+The text you get from L<POE::Component::IRC|POE::Component::IRC> is a raw
+byte string that has no inherent encoding. Most popular clients (mIRC, xchat,
+certain irssi configurations) encode their messages in Microsoft's CP1252
+encoding (their version of Latin-1) when the message fits only contains
+Latin-1 characters, otherwise falling back to UTF-8. Writing something like
+this to a file or a terminal is a recipe for disaster.
+
+This function takes a byte string (e.g. a message from an
+L<C<irc_public>|POE::Component::IRC/"irc_public"> handler) in "IRC encoding"
+and returns a UTF-8 encoded text string.
+
 =head2 C<irc_ip_get_version>
 
 Try to guess the IP version of an IP address.
