@@ -9,12 +9,12 @@ use POE qw(Driver::SysRW Filter::Line Filter::Stream
 use POE::Component::IRC::Plugin qw(:ALL);
 use Socket;
 
-our $VERSION = '1.1';
+our $VERSION = '6.08';
 
 use constant {
-    BLOCKSIZE          => 1024,   # Send DCC data in 1k chunks
-    INCOMING_BLOCKSIZE => 10_240, # 10k per DCC socket read
-    DCC_TIMEOUT        => 300,    # Five minutes for listening DCCs
+    OUT_BLOCKSIZE  => 1024,   # Send DCC data in 1k chunks
+    IN_BLOCKSIZE   => 10_240, # 10k per DCC socket read
+    LISTEN_TIMEOUT => 300,    # Five minutes for listening DCCs
 };
 
 sub new {
@@ -37,11 +37,11 @@ sub PCI_register {
                 _dcc_failed
                 _dcc_timeout
                 _dcc_up
-                _event_dcc
-                _event_dcc_accept
-                _event_dcc_chat
-                _event_dcc_close
-                _event_dcc_resume
+                _U_dcc
+                _U_dcc_accept
+                _U_dcc_chat
+                _U_dcc_close
+                _U_dcc_resume
             )],
         ],
     );
@@ -81,6 +81,33 @@ sub nataddr {
     return;
 }
 
+# returns information about a connection
+sub dcc_info {
+    my ($self, $id) = @_;
+    
+    if (!$self->{dcc}->{$id}) {
+        warn "dcc_info: Unknown wheel ID: $id\n";
+        return;
+    }
+
+    my %info;
+    @info{qw(nick type port file size done peeraddr)}
+        = @{ $self->{dcc}->{$id} }{qw(
+            nick type port file size done peeraddr
+        )};
+    return \%info;
+}
+
+sub _quote_file {
+    my ($file) = @_;
+
+    if ($file =~ /[\s"]/) {
+        $file =~ s|"|\\"|g;
+        $file = qq{"$file"};
+    }
+    return $file;
+}
+
 sub S_disconnected {
     my ($self) = $_;
     # clean up old cookies for any ignored RESUME requests 
@@ -90,10 +117,10 @@ sub S_disconnected {
 
 sub S_dcc_request {
     my ($self, $irc) = splice @_, 0, 2;
-    my ($nick, $type, $port, $cookie, $file, $size) = map { ref =~ /REF|SCALAR/ && ${ $_ } } @_;
+    my ($user, $type, $port, $cookie, $file, $size) = map { ref =~ /REF|SCALAR/ && ${ $_ } } @_;
+    my $nick = (split /!/, $user)[0];
 
     if ($type eq 'ACCEPT' && $self->{resuming}->{"$port+$nick"}) {
-
         # the old cookie has the peer's address
         my $old_cookie = delete $self->{resuming}->{"$port+$nick"};
         $irc->yield(dcc_accept => $old_cookie);
@@ -102,6 +129,7 @@ sub S_dcc_request {
         for my $cookie (values %{ $self->{dcc} }) {
             next if $cookie->{nick} ne $nick;
             next if $cookie->{port} ne $port;
+            $file = _quote_file($file);
             $irc->yield(ctcp => $nick => "DCC ACCEPT $file $port $size");
             last;
         }
@@ -110,57 +138,30 @@ sub S_dcc_request {
     return PCI_EAT_NONE;
 }
 
-# the U_* handlers are stubs which call our POE event handlers
-# so that we can do stuff related to our POE session, e.g.
-# create wheels and set alarms/delays
-
-sub U_dcc {
-    my ($self, $irc) = splice @_, 0, 2;
-    my @args = map { ref =~ /REF|SCALAR/ && ${ $_ } } @_;
-    $poe_kernel->call($self->{session_id}, _event_dcc => @args);
-    return PCI_EAT_NONE;
-}
-
-sub U_dcc_accept {
-    my ($self, $irc) = splice @_, 0, 2;
-    my @args = map { ref =~ /REF|SCALAR/ && ${ $_ } } @_;
-    $poe_kernel->call($self->{session_id}, _event_dcc_accept => @args);
-    return PCI_EAT_NONE;
-}
-
-sub U_dcc_chat {
-    my ($self, $irc) = splice @_, 0, 2;
-    my @args = map { ref =~ /REF|SCALAR/ && ${ $_ } } @_;
-    $poe_kernel->call($self->{session_id}, _event_dcc_chat => @args);
-    return PCI_EAT_NONE;
-}
-
-sub U_dcc_close {
-    my ($self, $irc) = splice @_, 0, 2;
-    my @args = map { ref =~ /REF|SCALAR/ && ${ $_ } } @_;
-    $poe_kernel->call($self->{session_id}, _event_dcc_close => @args);
-    return PCI_EAT_NONE;
-}
-
-sub U_dcc_resume {
-    my ($self, $irc) = splice @_, 0, 2;
-    my @args = map { ref && ${ $_ } } @_;
-    $poe_kernel->call($self->{session_id}, _event_dcc_resume => @args);
+# this is a stub handler for all U_dcc* events which redispatches them as
+# events to our own POE session so that we can do stuff related to it,
+# namely create wheels and set alarms/delays
+sub _default {
+    my ($self, $irc, $event) = splice @_, 0, 3;
+    return if $event !~ /^U_dcc(_accept|_chat|_close|_resume)?$/;
+    $event =~ s/^U_/_U_/;
+    my @args = map { $$_ } @_;
+    $poe_kernel->call($self->{session_id}, $event, @args);
     return PCI_EAT_NONE;
 }
 
 # Attempt to initiate a DCC SEND or CHAT connection with another person.
-sub _event_dcc {
+sub _U_dcc {
     my ($kernel, $self, $nick, $type, $file, $blocksize, $timeout)
         = @_[KERNEL, OBJECT, ARG0..$#_];
     
     if (!defined $type) {
-        warn "The 'dcc' event requires at least two arguments\n";
+        warn "The 'dcc' command requires at least two arguments\n";
         return;
     }
 
     my $irc = $self->{irc};
-    my ($bindport, $factory, $port, $myaddr, $size);
+    my ($bindport, $bindaddr, $factory, $port, $addr, $size);
 
     $type = uc $type;
     if ($type eq 'CHAT') {
@@ -168,27 +169,27 @@ sub _event_dcc {
     }
     elsif ($type eq 'SEND') {
         if (!defined $file) {
-            warn "Event 'dcc' requires three arguments for a SEND\n";
+            warn "The 'dcc' command requires three arguments for a SEND\n";
             return;
         }
         $size = (stat $file)[7];
         if (!defined $size) {
             $irc->send_event(
                 'irc_dcc_error',
-                0,
+                undef,
                 "Couldn't get ${file}'s size: $!",
                 $nick,
                 $type,
-                0,
+                undef,
                 $file,
             );
             return;
         }
     }
     
-    my $localaddr = $irc->localaddr;
-    if ($localaddr && $localaddr =~ tr/a-zA-Z.//) {
-        $localaddr = inet_aton($localaddr);
+    $bindaddr = $irc->localaddr();
+    if ($bindaddr && $bindaddr =~ tr/a-zA-Z.//) {
+        $bindaddr = inet_aton($bindaddr);
     }
 
     if ($self->{dccports}) {
@@ -200,48 +201,46 @@ sub _event_dcc {
     }
 
     $factory = POE::Wheel::SocketFactory->new(
-        BindAddress  => $localaddr || INADDR_ANY,
+        BindAddress  => $bindaddr || INADDR_ANY,
         BindPort     => $bindport,
         SuccessEvent => '_dcc_up',
         FailureEvent => '_dcc_failed',
         Reuse        => 'yes',
     );
     
-    ($port, $myaddr) = unpack_sockaddr_in($factory->getsockname());
-    $myaddr = inet_aton($self->{nataddr}) if $self->{nataddr};
+    ($port, $addr) = unpack_sockaddr_in($factory->getsockname());
+    $addr = inet_aton($self->{nataddr}) if $self->{nataddr};
   
-    if (!defined $myaddr) {
+    if (!defined $addr) {
         warn "dcc: Can't determine our IP address! ($!)\n";
         return;
     }
-    $myaddr = unpack 'N', $myaddr;
+    $addr = unpack 'N', $addr;
 
     my $basename = fileparse($file);
-    $basename = qq{"$basename"} if $basename =~ /[\s"]/;
+    $basename = _quote_file($basename);
 
     # Tell the other end that we're waiting for them to connect.
-    $irc->yield(ctcp => $nick => "DCC $type $basename $myaddr $port" . ($size ? " $size" : ''));
+    $irc->yield(ctcp => $nick => "DCC $type $basename $addr $port" . ($size ? " $size" : ''));
 
     # Store the state for this connection.
     $self->{dcc}->{ $factory->ID } = {
-        open       => undef,
-        nick       => $nick,
-        type       => $type,
-        file       => $file,
-        size       => $size,
-        port       => $port,
-        addr       => $myaddr,
-        done       => 0,
-        blocksize  => ($blocksize || BLOCKSIZE),
-        listener   => 1,
-        factory    => $factory,
-        listenport => $bindport,
-        clientaddr => $myaddr,
+        open      => 0,
+        nick      => $nick,
+        type      => $type,
+        file      => $file,
+        size      => $size,
+        port      => $port,
+        addr      => $addr,
+        done      => 0,
+        blocksize => ($blocksize || OUT_BLOCKSIZE),
+        listener  => 1,
+        factory   => $factory,
     };
     
     $kernel->alarm(
         '_dcc_timeout',
-        time() + ($timeout || DCC_TIMEOUT),
+        time() + ($timeout || LISTEN_TIMEOUT),
         $factory->ID,
     );
     
@@ -250,11 +249,11 @@ sub _event_dcc {
 
 # Accepts a proposed DCC connection to another client. See '_dcc_up' for
 # the rest of the logic for this.
-sub _event_dcc_accept {
+sub _U_dcc_accept {
     my ($self, $cookie, $myfile) = @_[OBJECT, ARG0, ARG1];
 
     if (!defined $cookie) {
-        warn "The 'dcc_accept' event requires at least one argument\n";
+        warn "The 'dcc_accept' command requires at least one argument\n";
         return;
     }
 
@@ -277,11 +276,11 @@ sub _event_dcc_accept {
 }
 
 # Send data over a DCC CHAT connection.
-sub _event_dcc_chat {
+sub _U_dcc_chat {
     my ($self, $id, @data) = @_[OBJECT, ARG0..$#_];
     
     if (!defined $id || !@data) {
-        warn "The 'dcc_chat' event requires at least two arguments\n";
+        warn "The 'dcc_chat' command requires at least two arguments\n";
         return;
     }
 
@@ -291,12 +290,12 @@ sub _event_dcc_chat {
     }
     
     if (!exists $self->{dcc}->{$id}->{wheel}) {
-        warn "dcc_chat: No DCC wheel for $id!\n";
+        warn "dcc_chat: No DCC wheel for id $id!\n";
         return;
     }
   
     if ($self->{dcc}->{$id}->{type} ne 'CHAT') {
-        warn "dcc_chat: $id isn't a DCC CHAT connection!\n";
+        warn "dcc_chat: id $id isn't associated with a DCC CHAT connection!\n";
         return;
     }
 
@@ -305,21 +304,28 @@ sub _event_dcc_chat {
 }
 
 # Terminate a DCC connection manually.
-sub _event_dcc_close {
+sub _U_dcc_close {
     my ($kernel, $self, $id) = @_[KERNEL, OBJECT, ARG0];
     my $irc = $self->{irc};
     
     if (!defined $id) {
-        warn "The 'dcc_close' event requires an id argument\n";
+        warn "The 'dcc_close' command requires an id argument\n";
         return;
     }
 
+    if (!exists $self->{dcc}->{$id}) {
+        warn "dcc_close: Unknown wheel ID: $id\n";
+        return;
+    }
+    
+    if (!exists $self->{dcc}->{$id}->{wheel}) {
+        warn "dcc_close: No DCC wheel for id $id!\n";
+        return;
+    }
+
+    # pending data, wait till it has been flushed
     if ($self->{dcc}->{$id}->{wheel}->get_driver_out_octets()) {
-        $kernel->delay_set(
-            '_event_dcc_close',
-            2,
-            \$id,
-        );
+        $kernel->delay_set(_U_dcc_close => 2, $id);
         return;
     }
 
@@ -327,15 +333,13 @@ sub _event_dcc_close {
         'irc_dcc_done',
         $id,
         @{ $self->{dcc}->{$id} }{qw(
-            nick type port file size
-            done listenport clientaddr
+            nick type port file size done peeraddr
         )},
     );
 
     # Reclaim our port if necessary.
-    if ($self->{dcc}->{$id}->{listener} && $self->{dccports}
-        && $self->{dcc}->{$id}->{listenport}) {
-        push ( @{ $self->{dccports} }, $self->{dcc}->{$id}->{listenport} );
+    if ($self->{dcc}->{$id}->{listener} && $self->{dccports}) {
+        push ( @{ $self->{dccports} }, $self->{dcc}->{$id}->{port} );
     }
 
     if (exists $self->{dcc}->{$id}->{wheel}) {
@@ -347,18 +351,15 @@ sub _event_dcc_close {
     return;
 }
 
-# bboett - first step - the user asks for a resume:
-# tries to resume a previous dcc transfer. See '_dcc_up' for
-# the rest of the logic for this.
 ## no critic (InputOutput::RequireBriefOpen)
-sub _event_dcc_resume {
+sub _U_dcc_resume {
     my ($self, $cookie, $myfile) = @_[OBJECT, ARG0, ARG1];
     my $irc = $self->{irc};
 
-    my $sender_file = $cookie->{file};
+    my $sender_file = _quote_file($cookie->{file});
     $cookie->{file} = $myfile if defined $myfile;
     my $size = -s $cookie->{file};
-    my $fraction = $size % INCOMING_BLOCKSIZE;
+    my $fraction = $size % IN_BLOCKSIZE;
     $size -= $fraction;
 
     # we need to truncate the whole thing, adjust the size we are
@@ -398,8 +399,7 @@ sub _dcc_read {
             'irc_dcc_get',
             $id,
             @{ $self->{dcc}->{$id} }{qw(
-                nick port file size
-                done listenport clientaddr
+                nick port file size done peeraddr
             )},
         );
     }
@@ -411,30 +411,25 @@ sub _dcc_read {
             'irc_dcc_send',
             $id,
             @{ $self->{dcc}->{$id} }{qw(
-                nick port file size done
-                listenport clientaddr
+                nick port file size done peeraddr
             )},
         );
 
         # Are we done yet?
         if ($self->{dcc}->{$id}->{done} >= $self->{dcc}->{$id}->{size}) {
             # Reclaim our port if necessary.
-            if ( $self->{dcc}->{$id}->{listener} && $self->{dccports}
-                && $self->{dcc}->{$id}->{listenport} ) {
-                push @{ $self->{dccports} },
-                    $self->{dcc}->{$id}->{listenport};
+            if ( $self->{dcc}->{$id}->{listener} && $self->{dccports}) {
+                push @{ $self->{dccports} }, $self->{dcc}->{$id}->{port};
             }
 
             $irc->send_event(
                 'irc_dcc_done',
                 $id,
                 @{ $self->{dcc}->{$id} }{qw(
-                    nick type port file size
-                    done listenport clientaddr
+                    nick type port file size done peeraddr
                 )},
             );
             delete $self->{wheelmap}->{ $self->{dcc}->{$id}->{wheel}->ID };
-            delete $self->{dcc}->{$id}->{wheel};
             delete $self->{dcc}->{$id};
             return;
         }
@@ -450,6 +445,7 @@ sub _dcc_read {
             $id,
             @{ $self->{dcc}->{$id} }{qw(nick port)},
             $data,
+            $self->{dcc}->{$id}->{peeraddr},
         );
     }
     
@@ -472,9 +468,8 @@ sub _dcc_failed {
     }
   
     # Reclaim our port if necessary.
-    if ( $self->{dcc}->{$id}->{listener} && $self->{dccports}
-        && $self->{dcc}->{$id}->{listenport} ) {
-        push ( @{ $self->{dccports} }, $self->{dcc}->{$id}->{listenport} );
+    if ( $self->{dcc}->{$id}->{listener} && $self->{dccports}) {
+        push ( @{ $self->{dccports} }, $self->{dcc}->{$id}->{port} );
     }
 
     DCC: {
@@ -486,13 +481,14 @@ sub _dcc_failed {
             if ($self->{dcc}->{$id}->{done} < $self->{dcc}->{$id}->{size}) {
                 last DCC;
             }
-      
+        }
+
+        if ($self->{dcc}->{$id}->{type} =~ /^(GET|CHAT)$/) {
             $irc->send_event(
                 'irc_dcc_done',
                 $id,
                 @{ $self->{dcc}->{$id} }{qw(
-                    nick type port file size
-                    done listenport clientaddr
+                    nick type port file size done peeraddr
                 )},
             );
 
@@ -501,21 +497,6 @@ sub _dcc_failed {
             }
             delete $self->{dcc}->{$id};
         }
-        elsif ($self->{dcc}->{$id}->{type} eq 'CHAT') {
-            $irc->send_event(
-                'irc_dcc_done',
-                $id,
-                @{ $self->{dcc}->{$id} }{qw(
-                    nick type port done
-                    listenport clientaddr
-                )}
-            );
-      
-            if ($self->{dcc}->{$id}->{wheel}) {
-                delete $self->{wheelmap}->{ $self->{dcc}->{$id}->{wheel}->ID };
-            }
-            delete $self->{dcc}->{$id};
-        }
         
         return;
     }
@@ -536,8 +517,7 @@ sub _dcc_failed {
         $id,
         $errstr,
         @{ $self->{dcc}->{$id} }{qw(
-            nick type port file size
-            done listenport clientaddr
+            nick type port file size done peeraddr
         )},
     );
 
@@ -569,22 +549,20 @@ sub _dcc_timeout {
 # This event occurs when a DCC connection is established.
 ## no critic (InputOutput::RequireBriefOpen)
 sub _dcc_up {
-    my ($kernel, $self, $sock, $addr, $port, $id) =
-        @_[KERNEL, OBJECT, ARG0 .. ARG3];
+    my ($kernel, $self, $sock, $peeraddr, $id) =
+        @_[KERNEL, OBJECT, ARG0, ARG1, ARG3];
     my $irc = $self->{irc};
     
-    # Monitor the new socket for incoming data
-    # and delete the listening socket.
+    # Delete the listening socket and monitor the accepted socket
+    # for incoming data
     delete $self->{dcc}->{$id}->{factory};
-    $self->{dcc}->{$id}->{addr} = $addr;
-    $self->{dcc}->{$id}->{clientaddr} = inet_ntoa($addr);
-    $self->{dcc}->{$id}->{port} = $port;
     $self->{dcc}->{$id}->{open} = 1;
+    $self->{dcc}->{$id}->{peeraddr} = inet_ntoa($peeraddr);
 
     $self->{dcc}->{$id}->{wheel} = POE::Wheel::ReadWrite->new(
         Handle => $sock,
         Driver => ($self->{dcc}->{$id}->{type} eq 'GET'
-            ? POE::Driver::SysRW->new( BlockSize => INCOMING_BLOCKSIZE )
+            ? POE::Driver::SysRW->new( BlockSize => IN_BLOCKSIZE )
             : POE::Driver::SysRW->new()
         ),
         Filter => ($self->{dcc}->{$id}->{type} eq 'CHAT'
@@ -627,12 +605,9 @@ sub _dcc_up {
     $irc->send_event(
         'irc_dcc_start',
         $id,
-        @{ $self->{dcc}->{$id} }{qw(nick type port)},
-        ($self->{dcc}->{$id}->{'type'} =~ /^(SEND|GET)$/
-            ? (@{ $self->{dcc}->{$id} }{qw(file size)})
-            : ()
-        ),
-        @{ $self->{dcc}->{$id} }{qw(listenport clientaddr)},
+        @{ $self->{dcc}->{$id} }{qw(
+            nick type port file size peeraddr
+        )},
     );
     
     return;
@@ -651,16 +626,17 @@ DCC transfers
  # send a file
  my $file = '/home/user/secret.pdf';
  my $recipient = 'that_guy';
- $irc->dcc($recipient => SEND => $file);
+ $irc->yield(dcc => $recipient => SEND => $file);
 
  # receive a file
  sub irc_dcc_request {
-     my ($user, $type, $port, $cookie, $file, $size) = @_[ARG0..$#_];
+     my ($user, $type, $port, $cookie, $file, $size, $addr) = @_[ARG0..$#_];
      return if $type ne 'SEND';
 
      my $irc = $_[SENDER]->get_heap();
      my $nick = (split /!/, $user)[0];
-     print "$nick wants to send me '$file' (size: $size) on port $port\n");
+
+     print "$nick wants to send me '$file' ($size bytes) from $addr:$port\n");
      $irc->yield(dcc_accept => $cookie);
  }
 
@@ -677,7 +653,7 @@ need to add it manually.
 Takes no arguments.
 
 Returns a plugin object suitable for feeding to
-L<POE::Component::IRC|POE::Component::IRC>'s C<plugin_add()> method.
+L<POE::Component::IRC|POE::Component::IRC>'s C<plugin_add> method.
 
 =head2 C<dccports>
 
@@ -688,6 +664,12 @@ an arrayref containing the port numbers.
 
 Sets the public NAT address to be used for DCC sends.
 
+=head2 C<dcc_info>
+
+Takes one argument, a DCC connection id (see below). Returns a hash of
+information about the connection. The keys are: B<'nick'>, B<'type'>,
+B<'port'>, B<'file'>, B<'size'>, B<'done,'>, and B<'peeraddr'>.
+
 =head1 COMMANDS
 
 The plugin responds to the following
@@ -696,20 +678,21 @@ L<POE::Component::IRC|POE::Component::IRC> commands.
 =head2 C<dcc>
 
 Send a DCC SEND or CHAT request to another person. Takes at least two
-arguments: the nick!user@host of the person to send the request to and the
-type of DCC request (SEND or CHAT). For SEND requests, be sure to add
-a third argument for the filename you want to send. Optionally, you
-can add a fourth argument for the DCC transfer blocksize, but the
-default of 1024 should usually be fine.
+arguments: the nickname of the person to send the request to and the type
+of DCC request (SEND or CHAT). For SEND requests, be sure to add a third
+argument for the filename you want to send. Optionally, you can add a fourth
+argument for the DCC transfer blocksize, but the default of 1024 should
+usually be fine. The fifth (and optional) argument is the request timeout
+value in seconds (default: 300).
 
 Incidentally, you can send other weird nonstandard kinds of DCCs too;
 just put something besides 'SEND' or 'CHAT' (say, 'FOO') in the type
-field, and you'll get back C<irc_dcc_*> events when activity happens
-on its DCC connection.
+field, and you'll get back C<irc_dcc_foo> events (with the same arguments as
+L<C<irc_dcc_chat>|/"irc_dcc_chat">) when data arrives on its DCC connection.
 
 If you are behind a firewall or Network Address Translation, you may want to
 consult L<POE::Component::IRC|POE::Component::IRC>'s
-L<C<connect>|POE::Component::IRC/"connect"> for some parameters that are
+L<C<connect>|POE::Component::IRC/"spawn"> for some parameters that are
 useful with this command.
 
 =head2 C<dcc_accept>
@@ -730,20 +713,19 @@ provides the name of the file to which you want to write.
 =head2 C<dcc_chat>
 
 Sends lines of data to the person on the other side of a DCC CHAT connection.
-Takes any number of arguments: the magic cookie from an
-L<C<irc_dcc_start>|/"irc_dcc_start"> event, followed by the data you wish to
-send. (It'll be separated by newlines for you.)
+The first argument should be the wheel id of the connection which you got
+from an L<C<irc_dcc_start>|/"irc_dcc_start"> event, followed by all the data
+you wish to send (it'll be separated with newlines for you).
 
 =head2 C<dcc_close>
 
 Terminates a DCC SEND or GET connection prematurely, and causes DCC CHAT
-connections to close gracefully. Takes one argument: the magic cookie
-from an L<C<irc_dcc_start>|/"irc_dcc_start"> or
-L<C<irc_dcc_request>|/"irc_dcc_request"> event.
+connections to close gracefully. Takes one argument: the wheel id of the
+connection which you got from an L<C<irc_dcc_start>|/"irc_dcc_start">
+(or similar) event.
 
 =head1 OUTPUT
 
-
 =head2 C<irc_dcc_request>
 
 B<Note:> This event is actually emitted by
@@ -756,19 +738,49 @@ and decide whether or not to accept it (with L<C<dcc_accept>|/"dcc_accept">)
 here. In the case of DCC SENDs, you can also request to resume the file with
 L<C<dcc_resume>|/"dcc_resume">.
 
-=over
+B<Note:> DCC doesn't provide a way to explicitly reject requests, so if you
+don't intend to accept one, just ignore it or send a
+L<NOTICE|POE::Component::IRC/"notice"> or L<PRIVMSG|POE::Component::IRC/"privmsg">
+to the peer explaining why you're not going to accept.
 
-=item ARG0: the peer's nickname
+=over 4
 
-=item ARG1: the port which the peer is listening on
+=item * C<ARG0>: the peer's nick!user@host
 
-=item ARG2: the DCC type
+=item * C<ARG1>: the DCC type (e.g. 'CHAT' or 'SEND')
 
-=item ARG3: this connection's "magic cookie"
+=item * C<ARG2>: the port which the peer is listening on
 
-=item ARG4: the file name
+=item * C<ARG3>: this connection's "magic cookie"
 
-=item ARG5: the file size (SEND only)
+=item * C<ARG4>: the file name (SEND only)
+
+=item * C<ARG5>: the file size (SEND only)
+
+=item * C<ARG6>: the IP address which the peer is listening on
+
+=back
+
+=head2 C<irc_dcc_start>
+
+This event notifies you that a DCC connection has been successfully
+established.
+
+=over 4
+
+=item * C<ARG0>: the connection's wheel id
+
+=item * C<ARG1>: the peer's nickname
+
+=item * C<ARG2>: the DCC type
+
+=item * C<ARG3>: the port number
+
+=item * C<ARG4>: the file name (SEND/GET only)
+
+=item * C<ARG5>: the file size (SEND/GET only)
+
+=item * C<ARG6>: the peer's IP address
 
 =back
 
@@ -777,79 +789,17 @@ L<C<dcc_resume>|/"dcc_resume">.
 Notifies you that one line of text has been received from the
 client on the other end of a DCC CHAT connection.
 
-=over
+=over 4
 
-=item ARG0: this connection's "magic cookie"
+=item * C<ARG0>: the connection's wheel id
 
-=item ARG1: the peer's nickname
+=item * C<ARG1>: the peer's nickname
 
-=item ARG2: the port number
+=item * C<ARG2>: the port number
 
-=item ARG3: the text they sent
+=item * C<ARG3>: the text they sent
 
-=back
-
-=head2 C<irc_dcc_done>
-
-You receive this event when a DCC connection terminates normally.
-Abnormal terminations are reported by L<C<irc_dcc_error>|/"irc_dcc_error">.
-
-=over
-
-=item ARG0: this connection's "magic cookie"
-
-=item ARG1: the peer's nickname
-
-=item ARG2: the DCC type
-
-=item ARG3: the port number
-
-=item ARG4: the filename
-
-=item ARG5: file size (SEND/GET only)
-
-=item ARG6: transferred file size (SEND/GET only, should be same as ARG5)
-
-=back
-
-=head2 C<irc_dcc_failed>
-
-You get this event when a DCC connection fails for some reason.
-
-=over
-
-=item ARG0: the operation that failed
-
-=item ARG1: the error number
-
-=item ARG2: the error string
-
-=item ARG3: this connection's "magic cookie"
-
-=back
-
-=head2 C<irc_dcc_error>
-
-You get this event whenever a DCC connection or connection attempt
-terminates unexpectedly or suffers some fatal error.
-
-=over
-
-=item ARG0: this connection's "magic cookie"
-
-=item ARG1: the error string
-
-=item ARG2: the peer's nickname
-
-=item ARG3: the DCC type
-
-=item ARG4: the port number
-
-=item ARG5: the file name
-
-=item ARG6: expected file size
-
-=item ARG7: tranferred file size
+=item * C<ARG4>: the peer's IP address
 
 =back
 
@@ -858,19 +808,21 @@ terminates unexpectedly or suffers some fatal error.
 Notifies you that another block of data has been successfully
 transferred from the client on the other end of your DCC GET connection.
 
-=over
+=over 4
 
-=item ARG0: this connection's "magic cookie"
+=item * C<ARG0>: the connection's wheel id
 
-=item ARG1: the peer's nickname
+=item * C<ARG1>: the peer's nickname
 
-=item ARG2: the port number
+=item * C<ARG2>: the port number
 
-=item ARG3: the file name
+=item * C<ARG3>: the file name
 
-=item ARG4: the file size
+=item * C<ARG4>: the file size
 
-=item ARG5: transferred file size
+=item * C<ARG5>: transferred file size
+
+=item * C<ARG6>: the peer's IP address
 
 =back
 
@@ -880,45 +832,80 @@ Notifies you that another block of data has been successfully
 transferred from you to the client on the other end of a DCC SEND
 connection.
 
-=over
+=over 4
 
-=item ARG0: this connection's "magic cookie"
+=item * C<ARG0>: the connection's wheel id
 
-=item ARG1: the peer's nickname
+=item * C<ARG1>: the peer's nickname
 
-=item ARG2: the port number
+=item * C<ARG2>: the port number
 
-=item ARG3: the file name
+=item * C<ARG3>: the file name
 
-=item ARG4: the file size
+=item * C<ARG4>: the file size
 
-=item ARG5: transferred file size
+=item * C<ARG5>: transferred file size
+
+=item * C<ARG6>: the peer's IP address
 
 =back
 
-=head2 C<irc_dcc_start>
+=head2 C<irc_dcc_done>
 
-This event notifies you that a DCC connection has been successfully
-established.
+You receive this event when a DCC connection terminates normally.
+Abnormal terminations are reported by L<C<irc_dcc_error>|/"irc_dcc_error">.
 
-=over
+=over 4
 
-=item ARG0: this connection's "magic cookie"
+=item * C<ARG0>: the connection's wheel id
 
-=item ARG1: the peer's nickname
+=item * C<ARG1>: the peer's nickname
 
-=item ARG2: the DCC type
+=item * C<ARG2>: the DCC type
 
-=item ARG3: the port number
+=item * C<ARG3>: the port number
 
-=item ARG4: the file name (SEND/GET only)
+=item * C<ARG4>: the filename (SEND/GET only)
 
-=item ARG5: the file size (SEND/GET only)
+=item * C<ARG5>: file size (SEND/GET only)
+
+=item * C<ARG6>: transferred file size (SEND/GET only)
+
+=item * C<ARG7>: the peer's IP address
+
+=back
+
+=head2 C<irc_dcc_error>
+
+You get this event whenever a DCC connection or connection attempt
+terminates unexpectedly or suffers some fatal error. Some of the
+following values might be undefined depending the stage at which
+the connection/attempt failed.
+
+=over 4
+
+=item * C<ARG0>: the connection's wheel id
+
+=item * C<ARG1>: the error string
+
+=item * C<ARG2>: the peer's nickname
+
+=item * C<ARG3>: the DCC type
+
+=item * C<ARG4>: the port number
+
+=item * C<ARG5>: the file name
+
+=item * C<ARG6>: file size in bytes
+
+=item * C<ARG7>: transferred file size in bytes
+
+=item * C<ARG8>: the peer's IP address
 
 =back
 
 =head1 AUTHOR
 
-Dennis 'C<fimmtiu>' Taylor, et al.
+Dennis 'C<fimmtiu>' Taylor and Hinrik E<Ouml>rn SigurE<eth>sson, hinrik.sig@gmail.com
 
 =cut
