@@ -4,9 +4,9 @@ use strict;
 use warnings;
 use Carp;
 use POE::Component::IRC::Plugin qw( :ALL );
-use POE::Component::IRC::Common qw( :ALL );
+use POE::Component::IRC::Common qw( matches_mask parse_user );
 
-our $VERSION = '5.90';
+our $VERSION = '6.08';
 
 BEGIN { 
     # Turn on the debugger's symbol source tracing
@@ -33,10 +33,6 @@ sub new {
 sub PCI_register {
     my ($self, $irc) = @_;
 
-    if (defined $self->{botowner} && !$irc->isa('POE::Component::IRC::State') ) {
-        die __PACKAGE__ . ' requires PoCo::IRC::State or a subclass thereof';
-    }
-
     $self->{irc} = $irc;
     $irc->plugin_register( $self, 'SERVER', qw(public msg) );
     
@@ -85,19 +81,19 @@ sub PCI_unregister {
 
 sub S_public {
     my ($self, $irc) = splice @_, 0 , 2;
-    my $nick = parse_user( ${ $_[0] } );
+    my $who     = ${ $_[0] };
     my $channel = ${ $_[1] }->[0];
-    my $what = ${ $_[2] };
-    my $me = $irc->nick_name();
+    my $what    = ${ $_[2] };
+    my $me      = $irc->nick_name();
 
-    my ($command) = $what =~ m/^\s*\Q$me\E[\:\,\;\.]?\s*(.*)$/i;
-    return PCI_EAT_NONE if !$command || !$self->_bot_owner($nick);
+    my ($command) = $what =~ m/^\s*\Q$me\E[:,;.!?~]?\s*(.*)$/i;
+    return PCI_EAT_NONE if !$command || !$self->_authed($who, $channel);
 
     my (@cmd) = split(/ +/, $command);
     my $cmd = uc (shift @cmd);
     
     if (defined $self->{commands}->{$cmd}) {
-        $self->{command}->{$cmd}->($self, 'privmsg', $channel, @cmd);
+        $self->{commands}->{$cmd}->($self, 'privmsg', $channel, @cmd);
     }
     
     return PCI_EAT_NONE;
@@ -105,32 +101,22 @@ sub S_public {
 
 sub S_msg {
     my ($self, $irc) = splice @_, 0 , 2;
-    my $nick = parse_user( ${ $_[0] } );
+    my $who     = ${ $_[0] };
+    my $nick    = parse_user($who);
     my $channel = ${ $_[1] }->[0];
     my $command = ${ $_[2] };
-    my (@cmd) = split(/ +/,$command);
-    my $cmd = uc (shift @cmd);
+    my (@cmd)   = split(/ +/,$command);
+    my $cmd     = uc (shift @cmd);
     
-    return PCI_EAT_NONE if !$self->_bot_owner($nick);
+    return PCI_EAT_NONE if !$self->_authed($who, $channel);
     
     if (defined $self->{commands}->{$cmd}) {
-        $self->{command}->{$cmd}->($self, 'notice', $nick, @cmd);
+        $self->{commands}->{$cmd}->($self, 'notice', $nick, @cmd);
     }
 
     return PCI_EAT_NONE;
 }
 
-#########################
-# Trust related methods #
-#########################
-
-sub _bot_owner {
-    my ($self, $nick) = @_;
-    $nick = $self->{irc}->nick_long_form($nick) if $nick !~ /!/;
-    return 1 if matches_mask( $self->{botowner}, $nick );
-    return;
-}
-
 ###############################
 # Plugin manipulation methods #
 ###############################
@@ -192,7 +178,7 @@ sub unload {
 
 sub _unload_subs {
     my $self = shift;
-    my $file = shift;
+    my $file = shift || return;
 
     for my $sym ( grep { index( $_, "$file:" ) == 0 } keys %DB::sub ) {
         eval { undef &$sym };
@@ -200,7 +186,7 @@ sub _unload_subs {
         delete $DB::sub{$sym};
     }
 
-    return $self;
+    return 1;
 }
 
 sub reload {
@@ -222,6 +208,14 @@ sub loaded {
     return keys %{ $self->{plugins} };
 }
 
+sub _authed {
+    my ($self, $who, $chan) = @_;
+
+    return $self->{auth_sub}->($self->{irc}, $who, $chan) if $self->{auth_sub};
+    return 1 if matches_mask($self->{botowner}, $who);
+    return;
+}
+
 1;
 __END__
 
@@ -274,18 +268,26 @@ code and a handy IRC interface.
 
 Takes two optional arguments:
 
-'botowner', an IRC mask to match against for people issuing commands via the
+B<'botowner'>, an IRC mask to match against for people issuing commands via the
 IRC interface;
+
+B<'auth_sub'>, a sub reference which will be called to determine if a user
+may issue commands via the IRC interface. Overrides B<'botowner'>. It will be
+called with three arguments: the IRC component object, the nick!user@host and
+the channel name as arguments. It should return a true value if the user is
+authorized, a false one otherwise.
  
-'debug', set to a true value to see when stuff goes wrong;
+B<'debug'>, set to a true value to see when stuff goes wrong;
 
-Not setting a 'botowner' effectively disables the IRC interface. 
+Not setting B<'botowner'> or B<'auth_sub'> effectively disables the IRC
+interface.
 
-If 'botowner' is specified the plugin checks that it is being loaded into a
-L<POE::Component::IRC::State> or sub-class and will fail to load otherwise.
+If B<'botowner'> is specified the plugin checks that it is being loaded into a
+L<POE::Component::IRC::State|POE::Component::IRC::State> or sub-class and will
+fail to load otherwise.
 
 Returns a plugin object suitable for feeding to
-L<POE::Component::IRC|POE::Component::IRC>'s plugin_add() method.
+L<POE::Component::IRC|POE::Component::IRC>'s C<plugin_add> method.
 
 =head2 C<load>
 
@@ -326,22 +328,23 @@ Returns a list of descriptors of managed plugins.
 
 =head1 INPUT
 
-An IRC interface is enabled by specifying a "botowner" mask to new(). Commands
-may be either invoked via a PRIVMSG directly to your bot or in a channel by
-prefixing the command with the nickname of your bot. One caveat, the parsing
-of the irc command is very rudimentary ( it merely splits the line on \s+ ). 
+An IRC interface is enabled by specifying a "botowner" mask to
+L<C<new>|/"new">. Commands may be either invoked via a PRIVMSG directly to
+your bot or in a channel by prefixing the command with the nickname of your
+bot. One caveat, the parsing of the irc command is very rudimentary (it
+merely splits the line on spaces).
 
 =head2 C<plugin_add>
 
-Takes the same arguments as load().
+Takes the same arguments as L<C<load>|/"load">.
 
 =head2 C<plugin_del>
 
-Takes the same arguments as unload().
+Takes the same arguments as L<C<unload>|/"unload">.
 
 =head2 C<plugin_reload>
 
-Takes the same arguments as reload().
+Takes the same arguments as L<C<reload>|/"reload">.
 
 =head2 C<plugin_loaded>
 
